local sr = KSR

local function startswith(self, str)
	return self:sub(1, #str) == str
end


function data_urlencode()

	local str = sr.pv.get("$var(push_caller_numb)");
	local number = url_encode(str);
	sr.pv.sets("$var(push_caller_numb)", number)

	local str = sr.pv.get("$var(push_calling_numb)");
	local number = url_encode(str);
	sr.pv.sets("$var(push_calling_numb)", number)

	local str = sr.pv.get("$var(push_caller_name)");
	local name = url_encode(str);
	sr.pv.sets("$var(push_caller_name)", name)

	local str = sr.pv.get("$var(push_calling_name)");
	local name = url_encode(str);
	sr.pv.sets("$var(push_calling_name)", name)

	local str = sr.pv.get("$var(push_message)");
	local name = url_encode(str);
	sr.pv.sets("$var(push_message)", name)

end


function url_encode(str)
	if (str) then
		str = string.gsub (str, "\n", "\r\n")
		str = string.gsub (str, "([^%w %-%_%.%~])",
		function (c) return string.format ("%%%02X", string.byte(c)) end)
		str = string.gsub (str, " ", "+")
	end
	return str
end


function parse_ice_candidates()

	local sdp = sr.pv.get("$rb")

	local media_type
	local candidates = {audio = {srflx = 0, tcprelay = 0, udprelay = 0},
						video = {srflx = 0, tcprelay = 0, udprelay = 0}}
	local candidates_lines = {}

	if type(sdp) == "string" then
		for line in string.gmatch(sdp, "(.-)\r?\n") do

			if startswith(line, "m=audio") then
				media_type = "audio"
				table.insert(candidates_lines, line)
			elseif startswith(line, "m=video") then
				media_type = "video"
				table.insert(candidates_lines, line)
			end

			local candidate_type
			if startswith(line, "a=candidate") then
				table.insert(candidates_lines, line)
				if line:find("srflx") then
					candidate_type = "srflx"
				elseif line:find("typ relay") then
					if line:find("127.0.0.1") then
						candidate_type = "tcprelay"
					else
						candidate_type = "udprelay"
					end
				end
			end

			if media_type and candidate_type then
				candidates[media_type][candidate_type] = candidates[media_type][candidate_type] + 1
			end
		end
	end

	local msg = string.format("%s/%s/%s/%s/%s/%s", candidates["audio"]["srflx"],
													candidates["audio"]["tcprelay"],
													candidates["audio"]["udprelay"],
													candidates["video"]["srflx"],
													candidates["video"]["tcprelay"],
													candidates["video"]["udprelay"])
	sr.pv.sets("$var(candidates)", msg)
	sr.pv.sets("$var(candidates_debug)", table.concat(candidates_lines, "\n"))
end


function add_ice_candidates_to_wms_network()
	local math = require"math"
	math.randomseed(os.time())

	local function gen_random_string()
		local chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
		local length = 16
		local randomString = ''

		local charTable = {}
		for c in chars:gmatch"." do
			table.insert(charTable, c)
		end

		for i = 1, length do
			randomString = randomString .. charTable[math.random(1, #charTable)]
		end

		return randomString
	end

	local sdp = sr.pv.gete("$avp(write_sdp_out)")
	local ips = sr.pv.gete("$sht(globals=>pbx_own_ips)")
	local ip_table = {}

	if type(ips) ~= "string" then
		return
	end

	local i = math.random(100000000,8999999999)
	for ip in string.gmatch(ips, '([^,]+)') do
		ip = ip:gsub("\"", "")
		table.insert(ip_table, {ip=ip, foundation=gen_random_string(), priority=i})
		i = i + math.random(10000000,59999999)
	end

	local res = ""
	for line in string.gmatch(sdp, "(.-)\r?\n") do
		if startswith(line, "a=candidate") then
			if line:find("0.0.0.0") then
				for _, v in pairs(ip_table) do
					local cand = line:gsub("a=candidate:(%w+) (%d) (%w+) (%d+) 0%.0%.0%.0", "a=candidate:"..v["foundation"].." %2 %3 "..v["priority"].." "..v["ip"])
					res = res..cand.."\r\n"
				end
			else
				res = res..line.."\r\n"
			end
		else
			res = res..line.."\r\n"
		end
	end
	if res ~= "" then
		sr.pv.sets("$var(sdp_wms_network)", res)
	end
end


function check_permissions(src_gr, dst_gr, what)
	sr.pv.sets("$var(check_permissions_result)", "")
	local license = sr.pv.get("$avp(licenseType)")
	local rv = require"common.permissions":check(src_gr, "yesno", what, dst_gr, license)
	sr.pv.sets("$var(check_permissions_result)", rv)
end
